%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%  Optimization for Game Approach 2 %%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% input: number of user time interval, user id, number of appliances
% appliances usage requirement, 
% time schedule, m/m usage, storage unit capacity, max discharging rate, 
% max charging rate, initial price vector
% output: load, appliance usage, discharge, charge, current battery 
function [l,x,dis,ch,cur,cost]=fopg2(m,T,id, napp, E, ta, mmgamma, ns, mdis, mch, A,lr,lb,li)

% 
clear tatemp;
tatemp=zeros(napp,T);
for i=1:napp;
   for k=1:T;
      tatemp(i,k)=ta(id,i,k);
   end
end
    


%% Initial output
l=zeros(T);
x=zeros(napp,T);
dis=zeros(T);
ch=zeros(T);
cur=zeros(T);

%% Optimization using CVX solver
cvx_begin quiet
% cvx_begin
%% Variables
variable xv(napp,T); % appliance usage schedule
variable disv(T); % discharging storage schedule
variable chav(T); % charging storage schedule

%% Auxiliary variables
% current capacity, auxiliary variables
expression curv(T); % current capacity
% starting with a full battery
curv(1)=ns(id); 
for k=2:T;
    curv(k)=curv(k-1)-disv(k)+chav(k); 
end

%load for each customer, auxiliary variables
expression lv(T);
for k=1:T;
    lv(k)=sum(xv(:,k))-disv(k)+chav(k);
end


% % total load
% expression Lg(T);
% Lg=sum(lv)+sum(lb,1)/2+sum(li,1)/2;


 %% objective function
    minimize(sum(A*lv)); % objective function
        
%     minimize(max(lv));
    
    %% constraints
    subject to
    % Each appliance uses more power than its minimum when operating
for j=1:napp;
    for k=1:T;
        xv(j,k)>=mmgamma(id,j,1)*ta(id,j,k);
    end
end

     % Each appliance uses les power than its maximum when operating
for j=1:napp;
    for k=1:T;
        xv(j,k)<=mmgamma(id,j,2)*ta(id,j,k);
    end
end   

    %appliance power usage requirement
    sum((tatemp.*xv),2)==E(id,:)';
    sum(xv,2)==E(id,:)';    
 
    % discharging and charging constraints
for k=1:T;
    disv(k)<=mdis(id);   % discharging rate
    chav(k)<=mch(id);   %charging rate
end
    
    % fully discharge daily
    sum(disv)==ns(id);
    %fully charge daily
    sum(chav)==ns(id);
    
    %current battery capacity >=0
    curv>=0;
    % current battery capacity<=storage capacity
    for k=1:T;
        curv(k)<=ns(id);
    end
        
    xv>=0; 
    disv>=0;
    chav>=0;   
    lv+ns(id)>=0;
     
cvx_end % end of optimization
    
%% Output
l=lv;
x=xv;
dis=disv;
ch=chav;
cur=curv;
cost=sum(A*l);