%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First created in Summer 2014  using Matlab 2013 %
% Requires CVX SOLVER http://cvxr.com/cvx/       %
% Feng Ye feng.ye@huskers.unl.edu,               %
% Summer 2014                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Feng Ye, Yi Qian and Rose Qingyang Hu, 
% "A Real-Time Information Based Demand-Side Management System in Smart Grid,"
% in IEEE Transactions on Parallel and Distributed Systems, vol. 27, no. 2, pp. 329-339, Feb. 1 2016.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% global optimal solution                       %
% P1: min max load                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all
clc


%% Initialization
T=8; %number of time intervals
Lg=zeros(T); % global load, auxiliary var
%% Residential customer
mr=3; % types of residential customers
nrApp=8; % number of appliances
mmgammar=zeros(mr,nrApp,2); % min/max power usage of an appliance
%xr=zeros(mr,nrApp); % appliance usage schedule, var
Er=zeros(mr,nrApp); % appliance load requirement, const
tar=zeros(mr,nrApp,T); % time schedule for appliances, const
nrs=zeros(mr,1); % storage unit
curStr=zeros(mr,T); % current capacity of storage unit, var
%disStr=zeros(mr,T); % discharging storage schedule, var
%chaStr=zeros(mr,T); % charging storage schedule, var
mdisr=zeros(mr,1); % max discharging rate of storage unit,const
mchr=zeros(mr,1); % max charging rate of storage unit, const
lr=zeros(mr,T); % load vector for each customer, auxiliary var
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Business customer
mb=2; % types of business customers
nbApp=8; % number of appliances
mmgammab=zeros(mb,nbApp,2); % min/max power usage of an appliance
%xr=zeros(mr,nrApp); % appliance usage schedule, var
Eb=zeros(mb,nbApp); % appliance load requirement, const
tab=zeros(mb,nbApp,T); % time schedule for appliances, const
nbs=zeros(mb,1); % storage unit
curStb=zeros(mb,T); % current capacity of storage unit, var
%disStb=zeros(mb,T); % discharging storage schedule, var
%chaStb=zeros(mb,T); % charging storage schedule, var
mdisb=zeros(mb,1); % max discharging rate of storage unit,const
mchb=zeros(mb,1); % max charging rate of storage unit, const
lb=zeros(mb,T); % load vector for each customer, auxiliary var
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Industrial customer
mi=2; % number of business customers
niApp=8; % number of appliances
mmgammai=zeros(mi,niApp,2); % min/max power usage of an appliance
%xr=zeros(mr,nrApp); % appliance usage schedule, var
Ei=zeros(mi,niApp); % appliance load requirement, const
tai=zeros(mi,niApp,T); % time schedule for appliances, const
nis=zeros(mi,1); % storage unit
curSti=zeros(mi,T); % current capacity of storage unit, var
%disSti=zeros(mi,T); % discharging storage schedule, var
%chaSti=zeros(mi,T); % charging storage schedule, var
mdisi=zeros(mi,1); % max discharging rate of storage unit,const
mchi=zeros(mi,1); % max charging rate of storage unit, const
li=zeros(mi,T); % load vector for each customer, auxiliary var
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Constants insertion
dt=24/T; %duration of an interval
%% Residential customer

Eavgr=[18,5,5,2,2,3,2,4
    18,3,3,1,1,3,2,2
    9,2,2,1,1,3,2,1];
% 
% Er=fE(Eavgr,mr,nrApp); % randomize appliance usage

Er=Eavgr;

% Appliance power requirement
% for i=1:mr;
i=1; %type 1 residential, big house
    for k=1:T;
        % EHPV charging
        if dt*k>=0 && dt*k<8
            tar(i,1,k)=1;
        else
            tar(i,1,k)=0;
        end
        % AC1
        if dt*k>=0 && dt*k<=24
            tar(i,2,k)=1;
        else
            tar(i,2,k)=0;
        end
        % AC2
        if dt*k>=14 && dt*k<18
            tar(i,3,k)=1;
        else
            tar(i,3,k)=0;
        end
        % Cooking
        if dt*k>=17 && dt*k<19
            tar(i,4,k)=1;
        else
            tar(i,4,k)=0;
        end
        % Dish washer
        if dt*k>=19 && dt*k<=24
            tar(i,5,k)=1;
        else
            tar(i,5,k)=0;
        end
        % Washing
        if dt*k>=20 && dt*k<=24
            tar(i,6,k)=1;
        else
            tar(i,6,k)=0;
        end
        % Electronics
        if dt*k>=18 && dt*k<=24
            tar(i,7,k)=1;
        else
            tar(i,7,k)=0;
        end
                        
        % Other appliances
        if dt*k>=0 && dt*k<=24
            tar(i,8,k)=1;
        else
            tar(i,8,k)=0;
        end
    end
% end

i=2; %type 1 residential, town house
    for k=1:T;
        % EHPV charging
        if dt*k>=0 && dt*k<8
            tar(i,1,k)=1;
        else
            tar(i,1,k)=0;
        end
        % AC1
        if dt*k>=0 && dt*k<=24
            tar(i,2,k)=1;
        else
            tar(i,2,k)=0;
        end
        % AC2
        if dt*k>=14 && dt*k<18
            tar(i,3,k)=1;
        else
            tar(i,3,k)=0;
        end
        % Cooking
        if dt*k>=17 && dt*k<19
            tar(i,4,k)=1;
        else
            tar(i,4,k)=0;
        end
        % Dish washer
        if dt*k>=19 && dt*k<23
            tar(i,5,k)=1;
        else
            tar(i,5,k)=0;
        end
        % Washing
        if dt*k>=20 && dt*k<23
            tar(i,6,k)=1;
        else
            tar(i,6,k)=0;
        end
        % Electronics
        if dt*k>=18 && dt*k<=24
            tar(i,7,k)=1;
        else
            tar(i,7,k)=0;
        end
                        
        % Other appliances
        if dt*k>=0 && dt*k<=24
            tar(i,8,k)=1;
        else
            tar(i,8,k)=0;
        end
    end
    
 i=3; %type 3 residential, apt
    for k=1:T;
        % EHPV charging
        if dt*k>=0 && dt*k<8
            tar(i,1,k)=1;
        else
            tar(i,1,k)=0;
        end
        % AC1
        if dt*k>=0 && dt*k<=24
            tar(i,2,k)=1;
        else
            tar(i,2,k)=0;
        end
        % AC2
        if dt*k>=14 && dt*k<18
            tar(i,3,k)=1;
        else
            tar(i,3,k)=0;
        end
        % Cooking
        if dt*k>=18 && dt*k<19
            tar(i,4,k)=1;
        else
            tar(i,4,k)=0;
        end
        % Dish washer
        if dt*k>=19 && dt*k<22
            tar(i,5,k)=1;
        else
            tar(i,5,k)=0;
        end

   %Washing & dryer
        if dt*k>=18 && dt*k<22
            tar(i,6,k)=1;
        else
            tar(i,6,k)=0;
        end
        % Electronics
        if dt*k>=18 && dt*k<=24
            tar(i,7,k)=1;
        else
            tar(i,7,k)=0;
        end
                        
        % Other appliances
        if dt*k>=0 && dt*k<=24
            tar(i,8,k)=1;
        else
            tar(i,8,k)=0;
        end
    end   
    

% min and max appliance usage
for i=1:mr;
    %EHPV
   mmgammar(i,1,1)= Er(i,1)/30;
   mmgammar(i,1,2)= Er(i,1)/3;
   %AC1
   mmgammar(i,2,1)= Er(i,2)/48;
   mmgammar(i,2,2)= Er(i,2)/10;
   %AC2
   mmgammar(i,3,1)= Er(i,3)/8;
   mmgammar(i,3,2)= Er(i,3)/2;
   %cooking
   mmgammar(i,4,1)= Er(i,4)/4;
   mmgammar(i,4,2)= Er(i,4)/0.9;
   %dish washer
   mmgammar(i,5,1)= Er(i,5)/8;
   mmgammar(i,5,2)= Er(i,5)/0.9;
   %Washing & dryer
   mmgammar(i,6,1)= Er(i,6)/8;
   mmgammar(i,6,2)= Er(i,6)/0.9;
   %Electronics
   mmgammar(i,7,1)= Er(i,7)/10;
   mmgammar(i,7,2)= Er(i,7)/5;
   %Others
   mmgammar(i,8,1)= Er(i,8)/48;
   mmgammar(i,8,2)= Er(i,8)/12;
end

mmgammar=dt*mmgammar; %update it to each time interval

%storage unit capacity
for i=1:mr;
   nrs(i)=sum(Er(i,:))/6; 
end

%max dis/charging rate
% for i=1:mr;
%     mdisr(i)=nrs(i)/3;
%     mchr(i)=nrs(i)/3;
% end
mdisr=nrs./3;
mchr=nrs./3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Business customer

Eavgb=10.*[90,50,50,10,40,10,20,10
    45,50,70,10,35,10,30,10];
Eb=Eavgb;
% for i=1:mb;
%     Eb(i,1)=10; % EHPV
%     Eb(i,2)=300; % AC
%     Eb(i,3)=200; % Computers
%     Eb(i,4)=100; % light
%     Eb(i,5)=100; % others
% end

% Appliance power requirement
i=1;% type 1
    for k=1:T;
        % EHPV charging
        if dt*k>=0 && dt*k<8
            tab(i,1,k)=1;
        else
            tab(i,1,k)=0;
        end
        % AC1
        if dt*k>=0 && dt*k<=24
            tab(i,2,k)=1;
        else
            tab(i,2,k)=0;
        end
        % AC2
        if dt*k>=8 && dt*k<18
            tab(i,3,k)=1;
        else
            tab(i,3,k)=0;
        end
        % Electronics1
        if dt*k>=0 && dt*k<=24
            tab(i,4,k)=1;
        else
            tab(i,4,k)=0;
        end
         % Electronics2
        if dt*k>=8 && dt*k<18
            tab(i,5,k)=1;
        else
            tab(i,5,k)=0;
        end
        % light1
        if dt*k>=0 && dt*k<=24
            tab(i,6,k)=1;
        else
            tab(i,6,k)=0;
        end
        % light2
        if dt*k>=8 && dt*k<18
            tab(i,7,k)=1;
        else
            tab(i,7,k)=0;
        end
        % Other appliances
        if dt*k>=0 && dt*k<=24
            tab(i,8,k)=1;
        else
            tab(i,8,k)=0;
        end
    end
    
i=2;% type 2
    for k=1:T;
        % EHPV charging
        if dt*k>=0 && dt*k<8
            tab(i,1,k)=1;
        else
            tab(i,1,k)=0;
        end
        % AC1
        if dt*k>=0 && dt*k<=24
            tab(i,2,k)=1;
        else
            tab(i,2,k)=0;
        end
        % AC2
        if dt*k>=10 && dt*k<20
            tab(i,3,k)=1;
        else
            tab(i,3,k)=0;
        end
        % Electronics1
        if dt*k>=0 && dt*k<=24
            tab(i,4,k)=1;
        else
            tab(i,4,k)=0;
        end
         % Electronics2
        if dt*k>=10 && dt*k<20
            tab(i,5,k)=1;
        else
            tab(i,5,k)=0;
        end
        % light1
        if dt*k>=0 && dt*k<=24
            tab(i,6,k)=1;
        else
            tab(i,6,k)=0;
        end
        % light2
        if dt*k>=10 && dt*k<20
            tab(i,7,k)=1;
        else
            tab(i,7,k)=0;
        end
        % Other appliances
        if dt*k>=0 && dt*k<=24
            tab(i,8,k)=1;
        else
            tab(i,8,k)=0;
        end
    end

% min and max appliance usage
for i=1:mb;
    % EHPV
   mmgammab(i,1,1)= Eb(i,1)/30;
   mmgammab(i,1,2)= Eb(i,1)/3;
   % AC1
   mmgammab(i,2,1)= Eb(i,2)/48;
   mmgammab(i,2,2)= Eb(i,2)/16;
   % AC2
   mmgammab(i,3,1)= Eb(i,3)/12;
   mmgammab(i,3,2)= Eb(i,3)/8;
   % Electronics1
   mmgammab(i,4,1)= Eb(i,4)/25;
   mmgammab(i,4,2)= Eb(i,4)/22;
   % Electronics2
   mmgammab(i,5,1)= Eb(i,5)/12;
   mmgammab(i,5,2)= Eb(i,5)/8;
   % light1
   mmgammab(i,6,1)= Eb(i,6)/26;
   mmgammab(i,6,2)= Eb(i,6)/20;
   % light2
   mmgammab(i,7,1)= Eb(i,7)/12;
   mmgammab(i,7,2)= Eb(i,7)/8;
   % others
   mmgammab(i,8,1)= Eb(i,8)/48;
   mmgammab(i,8,2)= Eb(i,8)/12;
end

mmgammab=dt*mmgammab; %update it to each time interval

%storage unit capacity
for i=1:mb;
   nbs(i)=sum(Eb(i,:))/6; 
end


%max dis/charging rate
% for i=1:mb;
%     mdisb(i)=nbs(i)/3;
%     mchb(i)=nbs(i)/3;
% end
mdisb=nbs./3;
mchb=nbs./3;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Industrial customer

Eavgi=[450,450,400,400,100,100,100,100
    450,450,500,500,500,100,100,100];
Ei=Eavgi;

% Appliance power requirement
i=1; % type 1, outdoot construction
    for k=1:T;
        % PHEV 1
        if dt*k>=0 && dt*k<8
            tai(i,1,k)=1;
        else
            tai(i,1,k)=0;
        end
        % PHEV 2
        if dt*k>=8 && dt*k<16
            tai(i,2,k)=1;
        else
            tai(i,2,k)=0;
        end
        % Machine 1
        if dt*k>=8 && dt*k<16
            tai(i,3,k)=1;
        else
            tai(i,3,k)=0;
        end
        %Machine 2
        if dt*k>=16 && dt*k<=24
            tai(i,4,k)=1;
        else
            tai(i,4,k)=0;
        end
         % Machine 3
        if dt*k>=0 && dt*k<=24
            tai(i,5,k)=1;
        else
            tai(i,5,k)=0;
        end
        % Electronics
        if dt*k>=0 && dt*k<=24
            tai(i,6,k)=1;
        else
            tai(i,6,k)=0;
        end
        % light
        if dt*k>=8 && dt*k<=24
            tai(i,7,k)=1;
        else
            tai(i,7,k)=0;
        end
        % Other appliances
        if dt*k>=0 && dt*k<=24
            tai(i,8,k)=1;
        else
            tab(i,8,k)=0;
        end
    end

    % Appliance power requirement
i=2; % type 2, indoor manufacturing
    for k=1:T;
        % PHEV 1
        if dt*k>=0 && dt*k<8
            tai(i,1,k)=1;
        else
            tai(i,1,k)=0;
        end
        % PHEV 2
        if dt*k>=8 && dt*k<16
            tai(i,2,k)=1;
        else
            tai(i,2,k)=0;
        end
        % Machine 1
        if dt*k>=8 && dt*k<16
            tai(i,3,k)=1;
        else
            tai(i,3,k)=0;
        end
        %Machine 2
        if dt*k>=16 && dt*k<=24
            tai(i,4,k)=1;
        else
            tai(i,4,k)=0;
        end
         % Machine 3
        if dt*k>=0 && dt*k<=24
            tai(i,5,k)=1;
        else
            tai(i,5,k)=0;
        end
        % Electronics
        if dt*k>=0 && dt*k<=24
            tai(i,6,k)=1;
        else
            tai(i,6,k)=0;
        end
        % light
        if dt*k>=8 && dt*k<=24
            tai(i,7,k)=1;
        else
            tai(i,7,k)=0;
        end
        % Other appliances
        if dt*k>=0 && dt*k<=24
            tai(i,8,k)=1;
        else
            tab(i,8,k)=0;
        end
    end



% min and max appliance usage
for i=1:mi;
    %PHEV2
   mmgammai(i,1,1)= Ei(i,1)/30;
   mmgammai(i,1,2)= Ei(i,1)/3;
   % PHEV2
   mmgammai(i,2,1)= Ei(i,2)/30;
   mmgammai(i,2,2)= Ei(i,2)/3;
   % Machine 1
   mmgammai(i,3,1)= Ei(i,3)/12;
   mmgammai(i,3,2)= Ei(i,3)/6;
   % Machine 2
   mmgammai(i,4,1)= Ei(i,4)/12;
   mmgammai(i,4,2)= Ei(i,4)/6;
   % Machine 3
   mmgammai(i,5,1)= Ei(i,5)/26;
   mmgammai(i,5,2)= Ei(i,5)/22;
   % Electronics
   mmgammai(i,6,1)= Ei(i,6)/26;
   mmgammai(i,6,2)= Ei(i,6)/22;
   % light2
   mmgammai(i,7,1)= Ei(i,7)/24;
   mmgammai(i,7,2)= Ei(i,7)/6;
   % others
   mmgammai(i,8,1)= Ei(i,8)/30;
   mmgammai(i,8,2)= Ei(i,8)/20;
end

mmgammai=dt*mmgammai; %update it to each time interval

%storage unit capacity
for i=1:mi;
   nis(i)=sum(Ei(i,:))/6; 
end

mdisi=nis./3;
mchi=nis./3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Optimization by cvx_solver
cvx_begin 
%% Variables
% Residential customer
variable xr(mr,nrApp,T); % appliance usage schedule
variable disStr(mr,T); % discharging storage schedule
variable chaStr(mr,T); % charging storage schedule

% Business customer
variable xb(mb,nbApp,T); % appliance usage schedule
variable disStb(mb,T); % discharging storage schedule
variable chaStb(mb,T); % charging storage schedule

% Industrial customer
variable xi(mi,niApp,T); % appliance usage schedule
variable disSti(mi,T); % discharging storage schedule
variable chaSti(mi,T); % charging storage schedule

%% Auxiliary variables
% Residential
% current capacity, auxiliary variables
expression curStr(mr,T); % current capacity
% starting with a full battery
for i=1:mr;
   curStr(i,1)=nrs(i); 
end
for i=1:mr;
    for k=2:T;
        curStr(i,k)=curStr(i,k-1)-disStr(i,k)+chaStr(i,k); 
    end
end

%load for each customer, auxiliary variables
expression lr(mr, T);
for i=1:mr;
    for k=1:T;
        lr(i,k)=sum(xr(i,:,k))-disStr(i,k)+chaStr(i,k);
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Business
% current capacity, auxiliary variables
expression curStb(mb,T); % current capacity
% starting with a full battery
for i=1:mb;
   curStb(i,1)=nbs(i); 
end
for i=1:mb;
    for k=2:T;
        curStb(i,k)=curStb(i,k-1)-disStb(i,k)+chaStb(i,k); 
    end
end

%load for each customer, auxiliary variables
expression lb(mb, T);
for i=1:mb;
    for k=1:T;
        lb(i,k)=sum(xb(i,:,k))-disStb(i,k)+chaStb(i,k);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Industrial
% current capacity, auxiliary variables
expression curSti(mi,T); % current capacity
% starting with a full battery
for i=1:mi;
   curSti(i,1)=nis(i); 
end
for i=1:mi;
    for k=2:T;
        curSti(i,k)=curSti(i,k-1)-disSti(i,k)+chaSti(i,k); 
    end
end

%load for each customer, auxiliary variables
expression li(mi, T);
for i=1:mi;
    for k=1:T;
        li(i,k)=sum(xi(i,:,k))-disSti(i,k)+chaSti(i,k);
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%Global load, auxiliary variables
expression Lg(T);
Lg=55.*sum(lr,1)+sum(lb,1)+sum(li,1);
% Lg=sum(lr,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 %% objective function
    minimize(max(Lg)); % objective function
    %% constraints
    subject to
    %% Residential
    % Each appliance uses more power than its minimum when operating
    for i=1:mr;
        for j=1:nrApp;
            for k=1;T;
            xr(i,j,k)>=mmgammar(i,j,1)*tar(i,j,k);
            end
        end
    end
     % Each appliance uses les power than its maximum when operating
    for i=1:mr;
        for j=1:nrApp;
            for k=1;T;
            xr(i,j,k)<=mmgammar(i,j,2)*tar(i,j,k);
            end
        end
    end 
    
    %appliance power usage requirement
    sum((tar.*xr),3)==Er;
    sum(xr,3)==Er;    
 
    % discharging and charging constraints
    for i=1:mr;
        for k=1:T;
            disStr(i,k)<=mdisr(i);   % discharging rate
            chaStr(i,k)<=mchr(i);   %charging rate
        end
    end

    
    % fully discharge daily
    sum(disStr,2)==nrs;
    %fully charge daily
    sum(chaStr,2)==nrs;
    
    %current battery capacity >=0
    curStr>=0;
    % current battery capacity<=storage capacity
    for i=1:mr;
        for k=1:T;
            curStr(i,k)<=nrs(i);
        end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %% Business
    % Each appliance uses more power than its minimum when operating
    for i=1:mb;
        for j=1:nbApp;
            for k=1;T;
            xb(i,j,k)>=mmgammab(i,j,1)*tab(i,j,k);
            end
        end
    end
     % Each appliance uses les power than its maximum when operating
    for i=1:mb;
        for j=1:nbApp;
            for k=1;T;
            xb(i,j,k)<=mmgammab(i,j,2)*tab(i,j,k);
            end
        end
    end 
    
    %appliance power usage requirement
    sum((tab.*xb),3)==Eb;
    sum(xb,3)==Eb;    
 
    % discharging and charging constraints
    for i=1:mb;
        for k=1:T;
            disStb(i,k)<=mdisb(i);   % discharging rate
            chaStb(i,k)<=mchb(i);   %charging rate
        end
    end

    
    % fully discharge daily
    sum(disStb,2)==nbs;
    %fully charge daily
    sum(chaStb,2)==nbs;
    
    %current battery capacity >=0
    curStb>=0;
    % current battery capacity<=storage capacity
    for i=1:mb;
        for k=1:T;
            curStb(i,k)<=nbs(i);
        end
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %% Industrial
    % Each appliance uses more power than its minimum when operating
    for i=1:mi;
        for j=1:niApp;
            for k=1;T;
            xi(i,j,k)>=mmgammai(i,j,1)*tai(i,j,k);
            end
        end
    end
     % Each appliance uses les power than its maximum when operating
    for i=1:mi;
        for j=1:niApp;
            for k=1;T;
            xi(i,j,k)<=mmgammai(i,j,2)*tai(i,j,k);
            end
        end
    end 
    
    %appliance power usage requirement
    sum((tai.*xi),3)==Ei;
    sum(xi,3)==Ei;    
 
    % discharging and charging constraints
    for i=1:mi;
        for k=1:T;
            disSti(i,k)<=mdisi(i);   % discharging rate
            chaSti(i,k)<=mchi(i);   %charging rate
        end
    end

    
    % fully discharge daily
    sum(disSti,2)==nis;
    %fully charge daily
    sum(chaSti,2)==nis;
    
    %current battery capacity >=0
    curSti>=0;
    % current battery capacity<=storage capacity
    for i=1:mi;
        for k=1:T;
            curSti(i,k)<=nis(i);
        end
    end
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%      
    
    xr>=0; 
    disStr>=0;
    chaStr>=0;   
        
    xb>=0; 
    disStb>=0;
    chaStb>=0;

    xi>=0; 
    disSti>=0;
    chaSti>=0;
cvx_end % end of optimization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% 
% %% plot
A=fcost(Lg);
costtot=sum(A.*Lg);
disp(costtot);
fplot(dt,lr,disStr,chaStr,curStr,lb,disStb,chaStb,curStb,li,disSti,chaSti,curSti,Lg);
